// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_deferred_widget_annotation/wyatt_deferred_widget_annotation.dart';
import 'package:wyatt_deferred_widget_annotation_example/widgets/my_awesome_widget1.dart'
    deferred as my_awesome_widget1;

class MyAwesomeWidget1Deferred extends StatefulWidget {
  const MyAwesomeWidget1Deferred({
    required this.color,
    this.placeholderBuilder,
    super.key,
  });

  final Color color;

  final Widget Function(BuildContext context)? placeholderBuilder;

  @override
  State<MyAwesomeWidget1Deferred> createState() =>
      MyAwesomeWidget1DeferredState();
}

class MyAwesomeWidget1DeferredState extends State<MyAwesomeWidget1Deferred> {
  static bool loaded = false;

  @override
  void initState() {
    load();
    super.initState();
  }

  Future<void> load() async {
    await my_awesome_widget1.loadLibrary();
    await Future.delayed(const Duration(seconds: 1));
    if (mounted) {
      setState(() {
        loaded = true;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    if (!loaded) {
      if (widget.placeholderBuilder != null) {
        return widget.placeholderBuilder!.call(context);
      }
      if (Deferred.placeholderBuilder != null) {
        return Deferred.placeholderBuilder!.call(context);
      }
      return const SizedBox.shrink();
    }
    return my_awesome_widget1.MyAwesomeWidget1(
      color: widget.color,
    );
  }
}
