// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_deferred_widget_annotation/wyatt_deferred_widget_annotation.dart';
import 'package:wyatt_deferred_widget_annotation_example/widgets/my_awesome_widget2.dart'
    deferred as my_awesome_widget2;

class MyAwesomeWidget2Deferred extends StatefulWidget {
  const MyAwesomeWidget2Deferred({
    this.placeholderBuilder,
    this.placeholderSize,
    super.key,
  });

  /// Widget builder for the placeholder.
  /// 
  /// Prior over the global placeholder defined in [Deferred].
  final Widget Function(BuildContext context)? placeholderBuilder;

  /// Size for the placeholder builder.
  /// 
  /// Useful to build a global placeholder defined in [Deferred] with
  /// different size depending of the widget.
  final Size? placeholderSize;

  @override
  State<MyAwesomeWidget2Deferred> createState() =>
      MyAwesomeWidget2DeferredState();
}

class MyAwesomeWidget2DeferredState extends State<MyAwesomeWidget2Deferred> {
  static bool loaded = false;

  @override
  void initState() {
    load();
    super.initState();
  }

  Future<void> load() async {
    await my_awesome_widget2.loadLibrary();
    await Future.delayed(const Duration(seconds: 1));
    if (mounted) {
      setState(() {
        loaded = true;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    if (!loaded) {
      if (widget.placeholderBuilder != null) {
        return SizedBox.fromSize(
          size: widget.placeholderSize,
          child: widget.placeholderBuilder!.call(context),
        );
      }
      if (Deferred.placeholderBuilder != null) {
        return SizedBox.fromSize(
          size: widget.placeholderSize,
          child: Deferred.placeholderBuilder!.call(context),
        );
      }
      return const SizedBox.shrink();
    }
    return my_awesome_widget2.MyAwesomeWidget2();
  }
}
